﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;
using System.IO;

namespace Proxer6Lib
{
    public  delegate void dOnCardRead(Proxer6 p, Card card);

    /// <summary>
    /// Card information
    /// </summary>
    public class Card
    {
        public string Number;
        public override string ToString()
        {
            return string.Format("Card\n\rNumber: {0}",Number);
        }
    }

    /// <summary>
    /// Reader manager
    /// </summary>
    public class Proxer6: IDisposable
    {
        /// <summary>
        /// Serial port
        /// </summary>
        SerialPort sp = null;

        /// <summary>
        /// Card read event
        /// </summary>
        public event dOnCardRead OnCardRead;

        /// <summary>
        /// Initialize reader communication
        /// </summary>
        /// <param name="ComPortNumber">com number</param>
        /// <returns>0 Success</returns>
        public int Init(uint ComPortNumber)
        {
             
            try
            {
                sp = new SerialPort("COM" + ComPortNumber, 115200);
                sp.Open();
                sp.DataReceived += new SerialDataReceivedEventHandler(sp_DataReceived);
            }
            catch (Exception ex)
            {
                return -1;
                throw ex;
            }


            return 0;
        }

        /// <summary>
        /// Data from reader
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sp_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            System.Diagnostics.Trace.TraceInformation("sp_DataReceived "+sp.BytesToRead);
            byte[] b= new byte[sp.BytesToRead];

            sp.Read(b, 0, sp.BytesToRead);
            ProcessData(b);
        }

        /// <summary>
        /// Processing received data
        /// </summary>
        /// <param name="data"></param>
        void ProcessData(byte[] data)
        {
            string sdata = System.Text.Encoding.ASCII.GetString(data);

            if (sdata.IndexOf("\r\n") > -1)
            {
                Card c = new Card() { Number = sdata.Substring(0, sdata.Length - 2) };
                if (OnCardRead != null) OnCardRead(this,c);
            }

        }
        /// <summary>
        /// Free resources (com port)
        /// </summary>
        public void Dispose()
        {
            if (sp != null)
            {
                sp.Close();
                sp.Dispose();
            }
            if (OnCardRead != null)
            {
                OnCardRead = null;
            }
        }

    }
}
